@@ -3,7 +3,7 @@
 *           D o u b l e - P r e c i s i o n    S p h e r e    C l a s s         *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 2004,2005 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 2004,2006 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXSphered.h,v 1.10.2.1 2006/03/21 07:08:29 fox Exp $                          *
+* $Id: FXSphered.h,v 1.17 2006/01/22 17:58:09 fox Exp $                         *
 ********************************************************************************/
 #ifndef FXSPHERED_H
 #define FXSPHERED_H
@@ -56,23 +56,36 @@ public:
   /// Assignment
   FXSphered& operator=(const FXSphered& sphere){ center=sphere.center; radius=sphere.radius; return *this; }
 
+  /// Set value from another sphere
+  FXSphered& set(const FXSphered& sphere){ center=sphere.center; radius=sphere.radius; return *this; }
+
+  /// Set value from center and radius
+  FXSphered& set(const FXVec3d& cen,FXdouble rad){ center=cen; radius=rad; return *this; }
+
+  /// Set value from center and radius
+  FXSphered& set(FXdouble x,FXdouble y,FXdouble z,FXdouble rad){ center.set(x,y,z); radius=rad; return *this; }
+
+  /// Comparison
+  bool operator==(const FXSphered& s) const { return center==s.center && radius==s.radius;}
+  bool operator!=(const FXSphered& s) const { return center!=s.center || radius!=s.radius;}
+
   /// Diameter of sphere
   FXdouble diameter() const { return radius*2.0; }
 
   /// Test if empty
-  FXbool empty() const { return radius<0.0; }
+  bool empty() const { return radius<0.0; }
 
   /// Test if sphere contains point x,y,z
-  FXbool contains(FXdouble x,FXdouble y,FXdouble z) const;
+  bool contains(FXdouble x,FXdouble y,FXdouble z) const;
 
   /// Test if sphere contains point p
-  FXbool contains(const FXVec3d& p) const;
+  bool contains(const FXVec3d& p) const;
 
   /// Test if sphere contains another box
-  FXbool contains(const FXRanged& box) const;
+  bool contains(const FXRanged& box) const;
 
   /// Test if sphere contains another sphere
-  FXbool contains(const FXSphered& sphere) const;
+  bool contains(const FXSphered& sphere) const;
 
   /// Include point
   FXSphered& include(FXdouble x,FXdouble y,FXdouble z);
@@ -80,26 +93,38 @@ public:
   /// Include point
   FXSphered& include(const FXVec3d& p);
 
+  /// Expand radius to include point
+  FXSphered& includeInRadius(FXdouble x,FXdouble y,FXdouble z);
+
+  /// Expand radius to include point
+  FXSphered& includeInRadius(const FXVec3d& p);
+
   /// Include given range into this one
   FXSphered& include(const FXRanged& box);
 
+  /// Expand radius to include box
+  FXSphered& includeInRadius(const FXRanged& box);
+
   /// Include given sphere into this one
   FXSphered& include(const FXSphered& sphere);
 
+  /// Expand radius to include sphere
+  FXSphered& includeInRadius(const FXSphered& sphere);
+
   /// Intersect sphere with normalized plane ax+by+cz+w; returns -1,0,+1
   FXint intersect(const FXVec4d& plane) const;
 
   /// Intersect sphere with ray u-v
-  FXbool intersect(const FXVec3d& u,const FXVec3d& v) const;
+  bool intersect(const FXVec3d& u,const FXVec3d& v) const;
 
   /// Test if box overlaps with sphere
-  friend FXAPI FXbool overlap(const FXRanged& a,const FXSphered& b);
+  friend FXAPI bool overlap(const FXRanged& a,const FXSphered& b);
 
   /// Test if sphere overlaps with box
-  friend FXAPI FXbool overlap(const FXSphered& a,const FXRanged& b);
+  friend FXAPI bool overlap(const FXSphered& a,const FXRanged& b);
 
   /// Test if spheres overlap
-  friend FXAPI FXbool overlap(const FXSphered& a,const FXSphered& b);
+  friend FXAPI bool overlap(const FXSphered& a,const FXSphered& b);
 
   /// Save object to a stream
   friend FXAPI FXStream& operator<<(FXStream& store,const FXSphered& sphere);
@@ -109,9 +134,9 @@ public:
   };
 
 
-extern FXAPI FXbool overlap(const FXRanged& a,const FXSphered& b);
-extern FXAPI FXbool overlap(const FXSphered& a,const FXRanged& b);
-extern FXAPI FXbool overlap(const FXSphered& a,const FXSphered& b);
+extern FXAPI bool overlap(const FXRanged& a,const FXSphered& b);
+extern FXAPI bool overlap(const FXSphered& a,const FXRanged& b);
+extern FXAPI bool overlap(const FXSphered& a,const FXSphered& b);
 
 extern FXAPI FXStream& operator<<(FXStream& store,const FXSphered& sphere);
 extern FXAPI FXStream& operator>>(FXStream& store,FXSphered& sphere);
